unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    ResolveButton: TButton;
    Panel2: TPanel;
    ResultMemo: TMemo;
    LaunchButton: TButton;
    AbortPipeButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure ResolveButtonClick(Sender: TObject);
    procedure LaunchButtonClick(Sender: TObject);
    procedure AbortPipeButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;



implementation

{$R *.DFM}

uses
  DUtils, AIOUSB;

function GetDeviceHandle(DeviceIndex: LongWord; pData: PHandle): LongWord; cdecl; external 'AIOUSB.dll';

const
  FILE_DEVICE_UNKNOWN = $0022;
  METHOD_BUFFERED = 0;
  METHOD_NEITHER  = 3;
  FILE_ANY_ACCESS = 0;

  IOCTL_ADAPT_INDEX = $0000;
  IOCTL_CY_BASE = LongWord(FILE_DEVICE_UNKNOWN shl 16) or (FILE_ANY_ACCESS shl 14);
  IOCTL_ADAPT_GET_DRIVER_VERSION        = IOCTL_CY_BASE or ((IOCTL_ADAPT_INDEX   ) shl 2) or METHOD_BUFFERED;
  IOCTL_ADAPT_SEND_EP0_CONTROL_TRANSFER = IOCTL_CY_BASE or ((IOCTL_ADAPT_INDEX+ 8) shl 2) or METHOD_BUFFERED;
  IOCTL_ADAPT_SELECT_INTERFACE          = IOCTL_CY_BASE or ((IOCTL_ADAPT_INDEX+ 3) shl 2) or METHOD_BUFFERED;
  IOCTL_ADAPT_ABORT_PIPE                = IOCTL_CY_BASE or ((IOCTL_ADAPT_INDEX+17) shl 2) or METHOD_BUFFERED;
  IOCTL_ADAPT_SEND_NON_EP0_DIRECT       = IOCTL_CY_BASE or ((IOCTL_ADAPT_INDEX+18) shl 2) or METHOD_NEITHER;

  AIOCyUSBClassGUID: TGUID = '{D52D27E3-CF76-25EB-72EA-596F2234C0E1}'; //(D1: $D52D27E3; D2: $CF76; D3: $25EB; D4: ($72, $EA, $59, $6F, $22, $34, $C0, $E1) );

  CyBaseTimeout = 42; //It's an integer, and all the .PDF says is it's the value you'd expect divided by 1000.

  TargetPipe = $86;

var
  DeviceIndex: LongWord;

type
  TThreadStrMethod = procedure (S: AnsiString) of object;

  TBulkThread = class(TThread)
  protected
    SyncMethod: TThreadStrMethod;
    SyncStr: AnsiString;

    procedure Execute; override;
    procedure DoSynchronize(Method: TThreadStrMethod; S: AnsiString); virtual;
    procedure DoSynchronize_Inner; virtual;
    procedure LogStr(S: AnsiString); virtual;
  end;

function BEStatus(bSucc: Boolean): LongWord;
begin
  if bSucc then
    Result := ERROR_SUCCESS
  else begin
    Result := GetLastError;
    if Result = ERROR_SUCCESS then Result := ERROR_GEN_FAILURE;
  end;
end;

{ TBulkThread }

procedure TBulkThread.DoSynchronize(Method: TThreadStrMethod; S: AnsiString);
begin
  SyncMethod := Method;
  SyncStr := S;
  Synchronize(DoSynchronize_Inner);
end;

procedure TBulkThread.DoSynchronize_Inner;
begin
  SyncMethod(SyncStr);
end;

procedure TBulkThread.Execute;
var
  Status, BytesRead: LongWord;
  ADData: array of array[Byte] of Word;

begin
  SetLength(ADData, 1);
  repeat
    DoSynchronize(LogStr, 'Reading');
    BytesRead := 0;
    Status := AWU_GenericBulkIn(DeviceIndex, TargetPipe, @ADData[0][0], SizeOf(ADData[0]), BytesRead);
    if Status <> ERROR_SUCCESS then begin
      DoSynchronize(LogStr, 'Error ' + IntToStr(Status) + 'd from AWU_GenericBulkIn.');
      Break;
    end;
    DoSynchronize(LogStr, 'Read ' + IntToHex(BytesRead, 1) + 'B.');
    if BytesRead = 0 then begin
      DoSynchronize(LogStr, 'Assuming that was an abort.');
      Break;
    end;
  until Terminated;
  DoSynchronize(LogStr, 'Bulk read thread done.');
end;

procedure TBulkThread.LogStr(S: AnsiString);
begin
  MainForm.ResultMemo.Lines.Add(S);
end;

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LoadDefaultFont(Font);

  ResolveButton.Click;
end;

procedure TMainForm.ResolveButtonClick(Sender: TObject);
var
  I: Integer;

begin
  DeviceIndex := diNone;

  for I := 0 to 31 do
    AIOUSB_CloseDevice(I)
  ;
  GetDevices;
  DeviceIndex := ResolveDeviceIndex(diOnly);

  if DeviceIndex = diNone then
    ResultMemo.Lines.Add('Device not found.')
  else
    ResultMemo.Lines.Add('Device found at ' + IntToHex(DeviceIndex, 1) + '.')
  ;
end;

procedure TMainForm.LaunchButtonClick(Sender: TObject);
begin
  ResultMemo.Lines.Add('Launching bulk read thread');

  TBulkThread.Create(False);
end;

procedure TMainForm.AbortPipeButtonClick(Sender: TObject);
var
  Status, BytesReturned: LongWord;
  hCyDevice: THandle;
  MyTargetPipe: Byte;

begin
  hCyDevice := 0;
  Status := GetDeviceHandle(DeviceIndex, @hCyDevice);
  if Status <> ERROR_SUCCESS then begin
    ResultMemo.Lines.Add('Error ' + IntToStr(Status) + 'd from GetDeviceHandle.');
    Exit;
  end;
  MyTargetPipe := TargetPipe;
  Status := BEStatus(DeviceIoControl(hCyDevice, IOCTL_ADAPT_ABORT_PIPE, @MyTargetPipe, SizeOf(MyTargetPipe), nil, 0, BytesReturned, nil));
  if Status <> ERROR_SUCCESS then begin
    ResultMemo.Lines.Add('Error ' + IntToStr(Status) + 'd from IOCTL_ADAPT_ABORT_PIPE.');
    //Exit;
  end;
end;

end.
